package com.api.games.service.impl;

import com.api.common.JSONUtils;
import com.api.core.response.Result;
import com.api.core.response.ResultGenerator;
import com.api.core.service.AbstractService;
import com.api.games.dao.GameAnswerMapper;
import com.api.games.dao.GameLogMapper;
import com.api.games.model.GameLog;
import com.api.games.service.GameLogService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * Created by wanghuiwen on 2020/02/23.
 */
@Service
@Transactional
public class GameLogServiceImpl extends AbstractService<GameLog> implements GameLogService {
    @Resource
    private GameLogMapper gameLogMapper;
    @Resource
    private GameAnswerMapper gameAnswerMapper;

    @Override
    public Result list(String search, String ordermap, Integer page, Integer size) {
        Map<String, Object> params = JSONUtils.json2map(search);
        Map<String, Object> order = JSONUtils.json2map(ordermap);

        for (String key : order.keySet()) {
            if (order.get(key) != null && order.get(key).equals("ascending")) order.put(key, "asc");
            if (order.get(key) != null && order.get(key).equals("descending")) order.put(key, "desc");
        }

        PageHelper.startPage(page, size);
        List<Map<String, Object>> res = gameLogMapper.list(params, order);
        PageInfo<Map<String, Object>> pageInfo = new PageInfo<>(res);
        return ResultGenerator.genSuccessResult(pageInfo);
    }

    @Override
    public Result detail(Integer id) {
//        List<Map<String, Object>> res = gameAnswerMapper.listByGameId(id);
//        Map<String, Object> params = new HashMap<>();
//        params.put("gameId",id);
//        Map<String, Object> statistics = gameAnswerMapper.statistics(params);
//        if(statistics == null ){
//            statistics = new HashMap<>();
//        }
//        statistics.put("list",res);

        return ResultGenerator.genSuccessResult(gameAnswerMapper.listByGameIdNew(id));
    }

    /**
     * 每天 早13：00 午：17：00；晚：21：00 执行
     *
     * @return
     */
    @Override
    public Result notPerformed() {
        Calendar calendar = Calendar.getInstance();
        int hour = calendar.get(Calendar.HOUR_OF_DAY);
        int type = 0;
        if (hour == 13) {
            type = 1;
        }
        if (hour == 17) {
            type = 2;
        }
        if (hour == 21) {
            type = 3;
        }
        if (type != 0) {
            List<GameLog> res = gameLogMapper.notPerformed(type);
            if (res != null && res.size() > 0) save(res);
        }
        return ResultGenerator.genSuccessResult();
    }

    @Override
    public List<Map<String, Object>> download(String search,String ordermap) {
        Map<String, Object> params = JSONUtils.json2map(search);
        Map<String, Object> order = JSONUtils.json2map(ordermap);

        for (String key : order.keySet()) {
            if (order.get(key) != null && order.get(key).equals("ascending")) order.put(key, "asc");
            if (order.get(key) != null && order.get(key).equals("descending")) order.put(key, "desc");
        }

        return gameAnswerMapper.list(params, order);
    }

    public Map<String,Object> download(String search) {
        Map<String, Object> params = JSONUtils.json2map(search);
        return gameAnswerMapper.statistics(params);
    }
}
